<?php

namespace jrosset\ExecutionTimer;

/**
 * A time duration in a unit
 */
class DurationUnit {
    const UNIT_MICROSECONDS = -2;
    const UNIT_MILLISECONDS = -1;
    const UNIT_SECONDS      = 0;
    const UNIT_MINUTES      = 1;
    const UNIT_HOURS        = 2;
    const UNIT_DAYS         = 3;

    /**
     * @var float Time duration, in the unit
     */
    private $duration;
    /**
     * @var int The time unit : self::UNIT_*
     */
    private $unit;
    /**
     * @var int The maximum value of time unit (e.g. maximum of 60 secondes). 0 if no maximum (UNIT_DAYS)
     */
    private $unit_max;

    /**
     * New time duration
     *
     * @param float $time Time duration, in the unit
     * @param int   $unit The time unit : self::UNIT_*
     */
    public function __construct ($time, $unit) {
        $this->duration = $time;
        $this->unit = $unit;

        $units_max = array(
            self::UNIT_MICROSECONDS => 1000,
            self::UNIT_MILLISECONDS => 1000,
            self::UNIT_SECONDS      => 60,
            self::UNIT_MINUTES      => 60,
            self::UNIT_HOURS        => 24,
            self::UNIT_DAYS         => 0,
        );
        $this->unit_max = $units_max[$unit];
    }

    /**
     * The time unit
     *
     * @return int The maximum value of time unit
     */
    public function getUnit () {
        return $this->unit;
    }
    /**
     * The maximum value of time unit
     *
     * E.g. maximum of 60 secondes<br>
     * 0 if no maximum (UNIT_DAYS)
     *
     * @return int The maximum value of time unit
     */
    public function getUnitMax () {
        return $this->unit_max;
    }

    /**
     * Get absolute time, with fractional
     *
     * @return float The absolute time, with fractional
     */
    public function asAbsoluteFactional () {
        return $this->duration;
    }
    /**
     * Get absolute time, without fractional
     *
     * @return int The absolute time, without fractional
     */
    public function asAbsoluteWhole () {
        return (int)floor($this->asAbsoluteFactional());
    }
    /**
     * Get relative time (to upper unit), with fractional
     *
     * @return float The relative time, with fractional
     */
    public function asRelativeFactional () {
        return $this->unit_max === 0 ? $this->asAbsoluteFactional() : $this->asAbsoluteFactional() % $this->unit_max;
    }
    /**
     * Get relative time (to upper unit), without fractional
     *
     * @return int The relative time, without fractional
     */
    public function asRelativeWhole () {
        return $this->unit_max === 0 ? $this->asAbsoluteWhole() : $this->asAbsoluteWhole() % $this->unit_max;
    }
}